#!/usr/bin/perl
use strict;
use POSIX qw(floor);
sub linear { my($u)=@_;
        if($u<=0.04045) { return $u/12.92; }
        return (($u+0.055)/1.055)**2.4;
}


my @img;
my $i=1;
for my $fn (glob("b???w???n*.p?m.tif")) {
	my($nix,$b,$w,$noise)=split /\.?[a-z]+/, $fn;
	$b=$b*1;
	$w=$w*1;
	my $bl=linear($b/255);
	my $wl=linear($w/255);
	my $d=$wl-$bl;
	my $noiseStep=floor(log($noise/$d*10)/log(2)+.5);
	push @img, {black=>$b, white=>$w, noiseStep=>$noiseStep, i=>$i, fn=>$fn};
#	print "$b $w $noise $noisestep $fn\n";
	$i++;
}

my %ks;
for my $img (@img) {
	# sauvola-0.025/OCR-D-N1_0001-BIN_sauvola.png
	my $best_k;
	my $max_psnr;
	for(my $k=0.025; $k<0.499; $k+=0.025) {
		my $sau=sprintf("sauvola-%.3f/OCR-D-N1_%04d-BIN_sauvola.png", $k, $img->{i});
		# b000w255n0.80000.pgm.tif gt-b000w191n0.41802.pgm
		my $gt ="gt-".$img->{fn};
		$gt=~s/\.tif$//;
		my $score=`compare -metric psnr $gt $sau x.bmp 2>&1`;
		if($score > $max_psnr) {
			$max_psnr=$score; $best_k=$k;
		}
	}
#	printf "%3d %3d %d %.3f %5.2f\n", $img->{black}, $img->{white}, $img->{noiseStep}, $best_k, $max_psnr;
	printf "%d %d %d %f %f\n", $img->{black}, $img->{white}, $img->{noiseStep}, $best_k, $max_psnr;
	$ks{"$img->{black} $img->{white} $img->{noiseStep}"}=$best_k;
}

for(my $ns=0; $ns<=3; $ns++) {
	print "noise level $ns\n   ";
	for my $w (64,128,191,255) {
		printf " %5d", $w;
	}
	print "\n";
	for my $b (0,64,128,191) {
		printf "%3d", $b;
		for my $w (64,128,191,255) {
			if(!$ks{"$b $w $ns"}) { printf "   -  "; next; }
			printf " %5.3f", $ks{"$b $w $ns"};
		}
		printf "\n";
	}
}
