#include <cstdio>
#include <cmath>
#include <cstdlib>
#include <random>

/* od -c Beethoven-testtext.pgm|head
0000000   P   5  \n   #       C   r   e   a   t   e   d       b   y    
0000020   G   I   M   P       v   e   r   s   i   o   n       2   .   1
0000040   0   .   1   8       P   N   M       p   l   u   g   -   i   n
0000060  \n   4   9   5   7       1   5   5   6  \n   2   5   5  \n 377
*/

double linear(double u) {
	if(u<=0.04045) return u/12.92;
	return pow((u+0.055)/1.055, 2.4);
}

double srgb(double u) {
	if(u<=0.0031308) return 12.92*u;
	return 1.055*pow(u, 1./2.4) -0.055;
}

int main(int argc, char **argv) {
	std::mt19937 gen;

	long unsigned width=4957;
	long unsigned height=1556;
	unsigned char *img;
	img=(unsigned char *)malloc(width*height);
	FILE *fr;
	fr=fopen("Beethoven-testtext.pgm", "r");
	if(fread(img, 1, 16*3+15, fr)!=16*3+15) return 255; // head
	if(fread(img, 1, width*height, fr)!=width*height) return 255;
	fclose(fr);

	for(double bl=0; bl<=0.75001; bl+=0.25) {
	double black=linear(bl);
	for(double wh=bl+0.25; wh<=1.001; wh+=0.25) {
	double white=linear(wh);
	for(int noiselevel=1; noiselevel<=4; noiselevel++) {
	// for(double noise=(white-black)/10; noise<(white-black)*0.8001; noise*=2) {
	double noise = (white-black)/10 * (1<<(noiselevel-1));
	std::normal_distribution<> d{0, noise};

	int ow=(width -3)/4;
	int oh=(height-3)/4;
	FILE *fw, *gt;
	char fn[80];
	snprintf(fn, 80, "b%03.0fw%03.0fn%d.ppm", bl*255, wh*255, noiselevel);
	fw=fopen(fn, "w");
	snprintf(fn, 80, "gt-b%03.0fw%03.0fn%d.pgm", bl*255, wh*255, noiselevel);
	gt=fopen(fn, "w");
	fprintf(fw, "P6 %d %d 255\n", ow, oh);
	fprintf(gt, "P5 %d %d 255\n", ow, oh);
	for(int y=0; y<oh; y++) {
	for(int x=0; x<ow; x++) {
		double sum=0;
		for(int yy=y*4; yy<y*4+4; yy++) {
		for(int xx=x*4; xx<x*4+4; xx++) {
			sum+=img[yy*width+xx]/255.;
		}}
		sum/=16;
		sum=sum*(white-black) +black;
		double rgbsum=0;
		for(int c=0; c<3; c++) {
			double s=sum+d(gen);
			if(s<0) s=0;
			if(s>1) s=1;
			switch(c) {
				case 0: rgbsum+=0.2126*s; break;
				case 1: rgbsum+=0.7152*s; break;
				case 2: rgbsum+=0.0722*s; break;
				default: return 255;
			}
			int v=srgb(s)*255 +.5;
			fprintf(fw, "%c", v);
		}
	  fprintf(gt, "%c", rgbsum > (black+white)/2 ? 255 : 0);
	}}
	fclose(fw);
	fclose(gt);

	}}}
}
